/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Annozilla.
 *
 * The Initial Developer of the Original Code is
 * Doug Daniels <rainking@alumni.rice.edu>
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  Matthew Wilson <matthew@mjwilson.demon.co.uk>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

function LocalServerUtils() {}

LocalServerUtils.NS_RDONLY      = 0x01;
LocalServerUtils.NS_WRONLY      = 0x02;
LocalServerUtils.NS_CREATE_FILE = 0x08;
LocalServerUtils.NS_TRUNCATE    = 0x20;

// user -- read, write; group -- read; other -- read
LocalServerUtils.DEFAULT_FILE_PERMS = 00400 | 00200 | 00040 | 00004;

LocalServerUtils.TEMPORARY_FILE_URI = "annozillaTempBody.html";

/**
 * Write a string into a temporary file into the user's profile area.
 * Note: if this is called multiple times, the same filename will be
 * used each time.
 * @param str String to write into file.
 * @return A 'file://' URI pointing at the file.
 */
LocalServerUtils.createTempURIFromString = function(str) {   
    try {
        // get the profile directory
        var fileLocator = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties);  
        var file = fileLocator.get("ProfD", Components.interfaces.nsIFile);
        
        // append temporary filename
        file.append(LocalServerUtils.TEMPORARY_FILE_URI);        
        
        // initialize local file
        var localFile = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
        localFile.initWithPath(file.path);
    
        // if it exists, delete it
        if (localFile.exists()) {
            localFile.remove(false);
        }
    
        // write flags
        var flags = LocalServerUtils.NS_WRONLY | LocalServerUtils.NS_CREATE_FILE | LocalServerUtils.NS_TRUNCATE;

        // get a file output stream
        var fileStream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
        fileStream.init(localFile, flags, LocalServerUtils.DEFAULT_FILE_PERMS, 0);

        fileStream.write(str, str.length);
        fileStream.close();

        // Deal with various versions of the Mozilla interfaces which do this
        if (Components.classes["@mozilla.org/network/io-service;1"] && Components.interfaces.nsIIOService) {

            var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
            var fileHandler = ioService.getProtocolHandler("file").QueryInterface(Components.interfaces.nsIFileProtocolHandler);

            // new version
            if (fileHandler.getURLSpecFromFile) {
                return fileHandler.getURLSpecFromFile(localFile);
            }
            // old version in ioService
            else {
                return ioService.getURLSpecFromFile(localFile);
            }
        }
        // really old version
        else {
            var file_url = Components.classes["@mozilla.org/network/standard-url;1"].createInstance(Components.interfaces.nsIFileURL);
            file_url.file = localFile;
            return file_url.spec;
        }

    }
    catch (e) {
        dump("Error writing out new file: " + e + "\n");
        return null;
    }
}
   
