/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Annozilla.
 *
 * The Initial Developer of the Original Code is
 * Matthew Wilson <matthew@mjwilson.demon.co.uk>.
 * Portions created by the Initial Developer are Copyright (C) 2002
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var gPostObserver;

/**
 * If Spellbound is installed, then unhide the spelling button.
 */
function setupSpellBound() {
    if (typeof(Components.
                classes["@mozilla.org/spellbound;1"]) != "undefined") {
        document.getElementById ("spellingButton").
            setAttribute ("style", "display: block");
        document.getElementById ("spellingButton").disabled = true;
    }
}

function onload() {
    try {
        if (window.arguments && (window.arguments.length == 4)) {
            gPostObserver = new PostObserver(window.arguments[0], window.arguments[1], window.arguments[2], window.arguments[3]);
            setupSpellBound ();
        }
        else {
            window.alert("Improper arguments passed to editor.xul");
            window.close();
        }
    } catch (e) {
        window.alert (e);
    }
}

/**
 * @param annozilla Annozilla object
 * @param annotation if updating, an annotation.  if new post, then null.
 * @param xptr if updating, then null.  if new post, then xptr string.
 */
function PostObserver(annotation, baseURL, xptr, bodyURI) {
  this.prefs = new AnnozillaPrefs(this);
  this.bundle = srGetStrBundle("chrome://annozilla/locale/annozilla.properties");

  this.annotation = annotation;
  this.baseURL = baseURL;
  this.xptr = xptr;

  var service = Components.classes[PostObserver.ANNOTATIONS_SERVICE_CONTRACTID].getService()
                    .QueryInterface(Components.interfaces.nsIAnnotationsService);
  this.app = service.registerApplication("annozilla");
  
  var type = null;
  var language = null;
  
  // if we're updating, grab the old information
  if (this.annotation) {
    // type
    type = this.annotation.data.getProperty(PostObserver.RDF_NS, "type");
    if (type) {
      // slice the namespace off the type
      var shortType = PostObserver.extractProperty(type);
      if (shortType) {
	type = shortType;
      }
    }
    
    // language
    language = this.annotation.data.getProperty(PostObserver.DC10_NS, "language");

    // creator
    this.creator = annotation.data.getProperty(PostObserver.DC10_NS, "creator");
  }
  // otherwise, if posting
  else {
      this.creator = this.prefs.getPref("creator");
  }
  
  // if we're not updating, or if the annotation didn't have the
  // necessary properties, set them to defaults
  if (!type) {
    type = PostObserver.DEFAULT_ANNOTATION_TYPE;
  }
  if (!language) {
    // try to get the langugage pref
    language = this.prefs.getPref("language");
    if (! language) {
      language = this.bundle.GetStringFromName("DEFAULT_LANGUAGE");
    }
  }
  
  if ( (!this.creator) || (this.creator == "") ) {
      this.creator = "Anonymous";
  }

  // set the actual XUL fields
  document.getElementById("annotationType").value = type;
  document.getElementById("annotationLanguage").value = language;
  document.getElementById("annotationAuthor").value = this.creator;
  document.getElementById("keepOpen").checked = this.prefs.getPref("keepEditWindowOpen");

  // updating
  if (this.annotation) {
      document.getElementById("post").label = "Update";
      this.setupServers(true, this.annotation.serverName);
  }
  // posting
  else {
      document.getElementById("post").label = "Post";
      this.setupServers(false, this.app.getDefaultPostServer());
  }
  
  // setup the HTML editor
  var editURI = bodyURI ? bodyURI : 'about:blank';
  this.editor = getEditorWrapper();
  this.editor.loadUrl(editURI);
  
  // setup spell-check
  if (! PostObserver.isSpellCheckerInstalled()) {
    this.hideItem("spellingButton");
  }
  else {
    this.setElementEnabled(document.getElementById("spellingButton"), true);
  }

  // add the postobserver as an annotations observer
  this.app.addObserver(this);

  // set a window closing observer
  var that = this;
  window.addEventListener("unload",
			  function() { that.windowClosing(); },
			  false);
}

PostObserver.DEFAULT_ANNOTATION_TYPE = "Comment";

PostObserver.PROFILE_DIRECTORY = "ProfD";
PostObserver.SERVERS_FILENAME = "annozillaServers.rdf";
PostObserver.SERVERS_ROOT_URN = "urn:annotations:servers";

PostObserver.ANNOTATIONS_SERVICE_CONTRACTID = "@mozilla.org/annotations_service;1";
PostObserver.XMLSERIALIZER_CONTRACTID = "@mozilla.org/xmlextras/xmlserializer;1";
PostObserver.METADATA_CONTRACTID = "@mozilla.org/annotation-metadata;1";

PostObserver.ANNOT_NS = "http://www.w3.org/2000/10/annotation-ns#";
PostObserver.RDF_NS   = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
PostObserver.DC10_NS    = "http://purl.org/dc/elements/1.0/";
PostObserver.DC11_NS    = "http://purl.org/dc/elements/1.1/";
PostObserver.THREAD_NS= "http://www.w3.org/2001/03/thread#";
PostObserver.XHTML_NS = "http://www.w3.org/1999/xhtml";
PostObserver.TYPE_NS = "http://www.w3.org/2000/10/annotationType#";

PostObserver.prototype = {
  editorShell: undefined,
  annozilla: undefined,
  annotation: undefined,
  xptr: undefined,

  // all nsIAnnotationObserver functions defined inline here
  autoLoadOnBeginPageLoad: function(url, win) {},
  autoLoadOnSuccessPageLoad: function(url, win) {},
  autoLoadOnFailPageLoad: function(url, win) {},
  autoLoadAnnotationsForServer: function(serverURI, win, annotatedURI, annotationsLength, annotations) {},
  autoLoadFailureForServer: function(serverURI, win, annotatedURI, message) {},
  deleteAnnotationOnSuccess: function(url) {
    // if we have deleted the annotation we were trying to update
    if (this.annotation && (this.annotation.annotationURI == url)) {
      window.alert("You have deleted the annotation you were editing.  The editing window " +
		   "will now close.");
      window.close();
      return;
    }
  },
  deleteAnnotationOnFailure: function(url, message) {},
  getRepliesOnSuccess: function(annotation, repliesLength, replies) {},
  getRepliesOnFailure: function(annotation, message) {},
  replyToAnnotationOnSuccess: function(original, reply) {},
  replyToAnnotationOnFailure: function(original, message) {},
  loadAnnotationsOnSuccess: function(serverURI, annotatedURI, annotationsLength, annotations) {},
  loadAnnotationsOnFailure: function(serverURI, annotatedURI, message) {},
  loadAnnotationBodyOnSuccess: function(annotation, bodyText) {},
  loadAnnotationBodyOnFailure: function(annotation, message) {},

  postAnnotationOnSuccess: function(newAnnotation) {

      document.getElementById("statusbar-text").setAttribute("label", "Posting...SUCCESS!");
      document.getElementById("posting").mode = "determined";

      var keepOpen = document.getElementById("keepOpen").checked;
      if (! keepOpen) {
	  window.close();
      }
      
  },

  postAnnotationOnFailure: function(message) {
      dump("postAnnotationOnFailure: " + message + "\n");

      document.getElementById("post").disabled = false;
      document.getElementById("cancel").disabled = false;

      document.getElementById("statusbar-text").setAttribute("label", "Posting...Failed.");
      document.getElementById("posting").mode = "determined";

      window.alert("Annozilla was unable to post the annotation.  The server responded: " + message + "\n");
  },

  updateAnnotationOnSuccess: function(updatedAnnotation) { 
      dump("updateAnnotationOnSuccess: " + updatedAnnotation.annotationURI + "\n");

      document.getElementById("statusbar-text").setAttribute("label", "Updating...SUCCESS!");
      document.getElementById("posting").mode = "determined";
      document.getElementById("post").disabled = false;
      document.getElementById("cancel").disabled = false;

      var keepOpen = document.getElementById("keepOpen").checked;
      if (! keepOpen) {
	  dump("trying to close window");
	  window.close();
	  return;
      }

  },
  updateAnnotationOnFailure: function(url, message) {
      dump("updateAnnotationOnFailure: " + url + ", " + message + "\n");
      dump("updateAnnotationOnSuccess: " + message + "\n");

      document.getElementById("statusbar-text").setAttribute("label", "Updating...Failed.");
      document.getElementById("posting").mode = "determined";
      document.getElementById("post").disabled = false;
      document.getElementById("cancel").disabled = false;

      window.alert("Annozilla was unable to update the annotation.  The server responded: " + message + "\n");
  }
};

PostObserver.prototype.setKeepOpen = function() {
    var prefs = Components.classes["@mozilla.org/preferences;1"].getService(Components.interfaces[Components.interfaces.nsIPref]);
    if ( prefs ) {
	prefs.SetBoolPref("annozilla.keepEditWindowOpen", document.getElementById("keepOpen").checked);
    }
}
	
/**
 * Splits a namespace + property string and 
 * returns the property.
 *
 * @param resourceString string of namespace + property
 * @return object with namespace and property fields
 */
PostObserver.extractProperty = function(resourceString) {
    var i;
    for (i = (resourceString.length - 1); i >= 0; i--) {
	var charAtI = resourceString.charAt(i);
	if (charAtI == '#' || charAtI == '/' || charAtI == ':') {
	    return resourceString.substring(i+1);
	}
    }
    
    // couldn't find it -- return null
    return null;
}

PostObserver.prototype.windowClosing = function() {
    this.app.removeObserver(this);
    this.prefs.destructor();
}

/**
 * @param onlyOneServerAllowed boolean whether only the defaultServer may be used
 * @param defaultServer string name of defaultServer to select, or null if none
 */
PostObserver.prototype.setupServers = function(onlyOneServerAllowed, defaultServer) {

    // get the preferences directory
    var fileLocator = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties);  
    var file = fileLocator.get("ProfD", Components.interfaces.nsIFile);
    
    // append servers filename
    file.append(PostObserver.SERVERS_FILENAME);	
    
    // get the data source
    var urlSpec = PostObserver.getURLSpecFromFile(file);
    var ds = gRDF.GetDataSourceBlocking(urlSpec);
    
    // intialize the listbox
    var list = document.getElementById("serverList");
    list.database.AddDataSource(ds);
    list.setAttribute("ref", PostObserver.SERVERS_ROOT_URN);

    if (defaultServer) {
	// this is the bad thing that happens when 
	// interface designers don't put length fields
	// on lists (bad mozilla, bad)
	for(var i=0; true; i++) {
	    list.selectedIndex = i;
	    if (list.value == defaultServer) {
		break;
	    }
	}
    }
    else {
	list.selectedIndex = 0;
    }

    if (onlyOneServerAllowed) {
	list.disabled = true;
    }
}
   

PostObserver.prototype.editorOnLoad = function(url, listener) {
  var editorShell = document.getElementById("edit-frame").editorShell;
  editorShell.editorType = 'html';
  editorShell.webShellWindow = window.frames[0];
  editorShell.contentWindow  = window.frames[0];
  editorShell.RegisterDocumentStateListener(listener);
  editorShell.contentsMIMEType = "text/html";
  editorShell.LoadUrl(url);
  
  return editorShell;
}

PostObserver.prototype.cancel = function() {
    window.close();
}

PostObserver.prototype.post = function() {

  var server = document.getElementById("serverList").value;
  // first off, ensure that this server is still around
  if (! this.app.hasServer(server)) {
    if (this.annotation) {
      window.alert("This annotation exists on the server " +
		   this.annotation.serverURI + ", which you have removed from the " + 
		   "server list in preferences.  To update this annotation, you must " + 
		   "add this server back to the list in preferences.");
    }
    else {
      window.alert("You have removed the selected server " + server + " from the " + 
		   "server list in preferences.  You must either add this server back to " + 
		   "the list in preferences or select a new server.");
    }
    return;
  }

  var xml = this.editor.getDocument();
/*
  // var s = new XMLSerializer();
  var s = Components.classes[PostObserver.XMLSERIALIZER_CONTRACTID].createInstance(Components.interfaces.nsIDOMSerializer);
  var xml = s.serializeToString(doc);
  
  // remove doctype
  xml = xml.replace (/<!DOCTYPE[^>]*>/,'');
  
  // remove _moz attributes
  xml = xml.replace (/ _moz_[a-z_]*="[^"]*"/g,'');

    // remove type="_moz" attribute
    xml = xml.replace (/type="_moz"/g,'');

    // lowercase the tag names
    xml = xml.replace (/<([a-zA-Z0-9\/]+)([^]*?)>/g, function(wholeMatch, tagName, restOfTag) { return "<" + tagName.toLowerCase() + restOfTag + ">"; });

    // add a namespace
    xml = xml.replace (/<html[^]*?>/, '<html xmlns="http://www.w3.org/1999/xhtml">');
*/

  var language = document.getElementById("annotationLanguage").value;
  var type = document.getElementById("annotationType").value;
  var date = PostObserver.formatDate();

  // animate progress bar
  document.getElementById("posting").mode = "undetermined";
  
  // disable the post and cancel buttons
  document.getElementById("post").disabled = true;
  document.getElementById("cancel").disabled = true;

  // updating
  if (this.annotation) {
    document.getElementById("statusbar-text").setAttribute("label", "Updating...");

    this.annotation.setBodyText(xml);
    this.annotation.data.setLiteralProperty(PostObserver.DC10_NS, "language", language);
    this.annotation.data.setResourceProperty(PostObserver.RDF_NS, "type", PostObserver.TYPE_NS + type);
    this.annotation.data.setLiteralProperty(PostObserver.DC10_NS, "date", date);

    this.app.updateAnnotation(this.annotation);
  }
  // posting
  else {
      // get character set for the window
      var charsetObj = window.QueryInterface(Components.interfaces.nsIDOMWindow)
      .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
      .getInterface(Components.interfaces.nsIDocCharset);
      var charset = charsetObj.charset;
      
    document.getElementById("statusbar-text").setAttribute("label", "Posting...");
    
    var data = Components.classes[PostObserver.METADATA_CONTRACTID].createInstance(Components.interfaces.nsIAnnotationMetadata);

    data.setRequiredProperties(PostObserver.TYPE_NS + type, this.baseURL, this.baseURL + "#" + this.xptr,
			       this.creator, date, date);
    data.setLiteralProperty(PostObserver.DC10_NS, "language", language);
    data.setResourceProperty(PostObserver.RDF_NS, "type", PostObserver.TYPE_NS + type);

    this.app.postAnnotation(data, xml, charset, server);
  }
}

PostObserver.prototype.spellCheck = function ()
{
    try {
        var args = [];
        args[0] = document.getElementById ("edit-frame").contentWindow.document.documentElement;
        window.openDialog("chrome://spellbound/content/SBSpellCheck.xul",
                "_blank", "chrome,close,titlebar,resizable,modal", false, true, true, args);

    } catch (e) {
        window.alert ("Unexpected error opening spelling checker: " + e);
    }
}

// from editor.js
PostObserver.isSpellCheckerInstalled = function() 
{ 
  return "@mozilla.org/spellchecker;1" in Components.classes;
} 


PostObserver.prototype.hideItem = function(id)
{
  var item = document.getElementById(id);
  if (item) {
    item.setAttribute("hidden", "true");
  }
}

// from editorUtilities.js
PostObserver.prototype.setElementEnabled = function (element, doEnable)
{
  if ( element ) {
    if ( doEnable ) {
      element.removeAttribute("disabled");
    }
    else {
      element.setAttribute("disabled", "true");
    }
  }
  else
  {
    dump("Element not found in SetElementEnabled\n");
  }
}


PostObserver.getURLSpecFromFile = function(file) {
    
    if (Components.classes["@mozilla.org/network/io-service;1"] && Components.interfaces.nsIIOService) {
	
	var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
	var fileHandler = ioService.getProtocolHandler("file").QueryInterface(Components.interfaces.nsIFileProtocolHandler);

	// new version
	if (fileHandler.getURLSpecFromFile) {
	    return fileHandler.getURLSpecFromFile(file);
	}
	// old version in ioService
	else {
	    return ioService.getURLSpecFromFile(file);
	}
    }
    // really old version
    else {
	var file_url = Components.classes["@mozilla.org/network/standard-url;1"].createInstance(Components.interfaces.nsIFileURL);
	file_url.file = file;
	return file_url.spec;
    }
}

PostObserver.formatDate = function() {
  var operandStr;

    // current time
    var now  = new Date(); 

    // regexp to add leading 0
    var reg = new RegExp(/^([0-9])$/); 

    // standard part of string common to all time zones
    var formatted = now.getUTCFullYear() + "-" +
    new String(now.getMonth() + 1).replace(reg, "0$1") + "-" +
    new String(now.getDate()).replace(reg, "0$1") + "T" +
    new String (now.getHours()).replace(reg, "0$1") + ":" +
    new String(now.getMinutes()).replace(reg, "0$1") + ":" +
    new String(now.getSeconds()).replace(reg, "0$1");

    // get timezone offset operandStr
    var offset = now.getTimezoneOffset();
    if (offset < 0) {
	operandStr = "+";
    }
    else if (offset > 0) {
	operandStr = "-";
    }
    // special case for +0 GMT
    else {
	return formatted + "Z";
    }

    // if offset less than zero, make it positive
    if (offset < 0) {
	offset *= -1;
    }

    // calculate hours and minutes
    var hours = Math.floor(offset / 60);
    var minutes = offset % 60;
    
    return formatted + operandStr + 
    new String(hours).replace(reg, "0$1") + ":" +
    new String(minutes).replace(reg, "0$1");
}

function getEditorWrapper () {
   return new MidasEditorWrapper();
}

function MidasEditorWrapper () {
}

MidasEditorWrapper.prototype.loadUrl = function (url) {
    /* In Firefox 1.5, we can't load annotations with content type
       application/xhtml+xml. So load the annotation body using XMLHttpRequest
       and write the result into the editing frame. */
    var bodyLoader = new XMLHttpRequest();
    bodyLoader.open ("GET", url);
    bodyLoader.onload = function () {
        document.getElementById ("edit-frame").contentWindow.document.write (bodyLoader.responseText);
        document.getElementById ("edit-frame").contentWindow.document.close ();
        designModeOn();
    };
    bodyLoader.onerror = function () { 
        window.alert ("Sorry, couldn't download the annotation body");
    };
    bodyLoader.send ('');
} 

function designModeOn () {
    try {
        document.getElementById ("edit-frame").contentWindow.document.designMode = "on";
        document.getElementById ("boldButton").disabled = false;
        document.getElementById ("italicButton").disabled = false;
        document.getElementById ("underlineButton").disabled = false;
        document.getElementById ("ulButton").disabled = false;
        document.getElementById ("olButton").disabled = false;
        document.getElementById ("spellingButton").disabled = false;
        document.getElementById ("post").disabled = false;
    } catch (e) {
        window.alert ("There was an error setting up the editor: " + e);
    }
}

// Carry out an editor command
function simpleCommand (command) {
    document.getElementById("edit-frame").contentWindow.document.execCommand(command, false, null);
}

MidasEditorWrapper.prototype.getDocument = function () {
    var serializer = new XMLSerializer();
    var serialized = serializer.serializeToString (document.getElementById("edit-frame").contentWindow.document);
    // remove doctype
    serialized = serialized.replace (/<!DOCTYPE[^>]*>/,'');

    // This is hacky. Why do we need to lower-casing of XHTML elements?
    // (This doesn't even do attributes...)
    serialized = serialized.replace
        ( /<(\/?\w+)/g,
          function (s1) {
          return s1.toLowerCase();
          }
        );

    // remove _moz attributes
    serialized = serialized.replace (/\s_moz_[a-z_]*="[^"]*"/g,'');

    // remove type="_moz" attribute
    serialized = serialized.replace (/type="_moz"/g,'');

    // Ensure namespace on HTML element
    serialized = serialized.replace (/<html[^>]*?>/, '<html xmlns="http://www.w3.org/1999/xhtml">');
  
    return serialized;
}
